#include <iostream>
#include <cmath>

using namespace std;

class qF
{
public:
   double operator()( double x ) const
   {  return( x - log(x) - 2 );  } 
}; // qF
      
class cosF
{
public:
   double operator()( double x ) const
   {  return cos( x );  } 
}; // cosF

template < class Func >
void bisection( double a0, double b0, double epsilon,
                double& root, int& errCode );

int main()
{
   const double pi( 4*atan(1.0) );
   double riza;
   int   errCode;

// bisection( &q, 0.1, 1.0, 1e-5, riza, errCode );
   bisection< qF >( 0.1, 1.0, 1e-5, riza, errCode );
   if ( errCode != 0 )  cout << "o " << endl;
                  else  cout << "  = " << riza << endl;
// bisection( &cos, 0.0, pi, 1e-5, riza, errCode );
   bisection< cosF >( 0.0, pi, 1e-5, riza, errCode );
   if ( errCode != 0 )  cout << "o " << endl;
                  else  cout << "  = " << riza << endl;
} // main

// bisection -- o    f(r) = 0 o
//               [a,b]   oo  o.
//               f(a)*f(b) > 0  errCode = 1
//              : |root - r| < epsilon  errCode = 0.
template < class Func >
void bisection( double a, double b, double epsilon,
                double& root, int& errCode )
{
   Func   f;
   double m;

   if ( f(a)*f(b) > 0 )
      errCode = 1;
   else
   {
      while ( fabs(b - a)/2 >= epsilon )
      {
         m = (a + b) / 2;
         if ( f(a)*f(m) <= 0.0 )  b = m;
                            else  a = m;
      } // while
      root = m;
      errCode = 0;
   } // if
} // bisection
